@@ -6,14 +6,12 @@
 from __future__ import print_function
 
 import imp
-import sys
 import warnings
 from contextlib import contextmanager
-from types import ModuleType
 
 import zmq
 
-from ipykernel import pickleutil
+from .. import serialize
 from traitlets import (
     HasTraits, Any, Bool, List, Dict, Set, Instance, CFloat, Integer
 )
@@ -57,12 +55,6 @@ def sync_results(f, self, *args, **kwargs):
         self._sync_results()
     return ret
 
-@decorator
-def spin_after(f, self, *args, **kwargs):
-    """call spin after the method."""
-    ret = f(self, *args, **kwargs)
-    self.spin()
-    return ret
 
 #-----------------------------------------------------------------------------
 # Classes
@@ -98,11 +90,11 @@ class View(HasTraits):
 
     """
     # flags
-    block=Bool(False)
-    track=Bool(True)
+    block = Bool(False)
+    track = Bool(False)
     targets = Any()
 
-    history=List()
+    history = List()
     outstanding = Set()
     results = Dict()
     client = Instance('ipyparallel.Client', allow_none=True)
@@ -117,6 +109,7 @@ class View(HasTraits):
         super(View, self).__init__(client=client, _socket=socket)
         self.results = client.results
         self.block = client.block
+        self.executor = ViewExecutor(self)
 
         self.set_flags(**flags)
 
@@ -201,7 +194,7 @@ class View(HasTraits):
         delta = self.outstanding.difference(self.client.outstanding)
         completed = self.outstanding.intersection(delta)
         self.outstanding = self.outstanding.difference(completed)
-        
+    
     @sync_results
     @save_ids
     def _really_apply(self, f, args, kwargs, block=None, **options):
@@ -226,7 +219,6 @@ class View(HasTraits):
         """
         return self._really_apply(f, args, kwargs, block=False)
 
-    @spin_after
     def apply_sync(self, f, *args, **kwargs):
         """calls ``f(*args, **kwargs)`` on remote engines in a blocking manner,
          returning the result.
@@ -301,8 +293,7 @@ class View(HasTraits):
             targets = self.targets
         return self.client.shutdown(targets=targets, restart=restart, hub=hub, block=block)
 
-    @spin_after
-    def get_result(self, indices_or_msg_ids=None, block=None, owner=True):
+    def get_result(self, indices_or_msg_ids=None, block=None, owner=False):
         """return one or more results, specified by history index or msg_id.
 
         See :meth:`ipyparallel.client.client.Client.get_result` for details.
@@ -500,16 +491,19 @@ class DirectView(View):
         
         adds support for closures, etc.
         
-        This calls ipykernel.pickleutil.use_dill() here and on each engine.
+        This calls ipyparallel.serialize.use_dill() here and on each engine.
         """
-        pickleutil.use_dill()
-        return self.apply(pickleutil.use_dill)
+        print(serialize)
+        serialize.use_dill()
+        return self.apply(serialize.use_dill)
 
     def use_cloudpickle(self):
         """Expand serialization support with cloudpickle.
+        
+        This calls ipyparallel.serialize.use_cloudpickle() here and on each engine.
         """
-        pickleutil.use_cloudpickle()
-        return self.apply(pickleutil.use_cloudpickle)
+        serialize.use_cloudpickle()
+        return self.apply(serialize.use_cloudpickle)
 
 
     @sync_results
@@ -552,20 +546,18 @@ class DirectView(View):
         targets = self.targets if targets is None else targets
         
         _idents, _targets = self.client._build_targets(targets)
-        msg_ids = []
-        trackers = []
+        futures = []
         for ident in _idents:
-            msg = self.client.send_apply_request(self._socket, f, args, kwargs, track=track,
+            future = self.client.send_apply_request(self._socket, f, args, kwargs, track=track,
                                     ident=ident)
-            if track:
-                trackers.append(msg['tracker'])
-            msg_ids.append(msg['header']['msg_id'])
+            futures.append(future)
+        if track:
+            trackers = [f.tracker for f in futures]
+        else:
+            trackers = []
         if isinstance(targets, int):
-            msg_ids = msg_ids[0]
-        tracker = None if track is False else zmq.MessageTracker(*trackers)
-        ar = AsyncResult(self.client, msg_ids, fname=getname(f), targets=_targets,
-            tracker=tracker, owner=True,
-        )
+            futures = futures[0]
+        ar = AsyncResult(self.client, futures, fname=getname(f), targets=_targets, owner=True)
         if block:
             try:
                 return ar.get()
@@ -637,17 +629,17 @@ class DirectView(View):
         targets = self.targets if targets is None else targets
 
         _idents, _targets = self.client._build_targets(targets)
-        msg_ids = []
-        trackers = []
+        futures = []
         for ident in _idents:
-            msg = self.client.send_execute_request(self._socket, code, silent=silent, ident=ident)
-            msg_ids.append(msg['header']['msg_id'])
+            future = self.client.send_execute_request(self._socket, code, silent=silent, ident=ident)
+            futures.append(future)
         if isinstance(targets, int):
-            msg_ids = msg_ids[0]
-        ar = AsyncResult(self.client, msg_ids, fname='execute', targets=_targets, owner=True)
+            futures = futures[0]
+        ar = AsyncResult(self.client, futures, fname='execute', targets=_targets, owner=True)
         if block:
             try:
                 ar.get()
+                ar.wait_for_output()
             except KeyboardInterrupt:
                 pass
         return ar
@@ -720,7 +712,6 @@ class DirectView(View):
         """
         block = block if block is not None else self.block
         targets = targets if targets is not None else self.targets
-        applier = self.apply_sync if block else self.apply_async
         if isinstance(names, string_types):
             pass
         elif isinstance(names, (list,tuple,set)):
@@ -744,7 +735,7 @@ class DirectView(View):
 
         mapObject = Map.dists[dist]()
         nparts = len(targets)
-        msg_ids = []
+        futures = []
         trackers = []
         for index, engineid in enumerate(targets):
             partition = mapObject.getPartition(seq, index, nparts)
@@ -753,18 +744,10 @@ class DirectView(View):
             else:
                 ns = {key: partition}
             r = self.push(ns, block=False, track=track, targets=engineid)
-            msg_ids.extend(r.msg_ids)
-            if track:
-                trackers.append(r._tracker)
+            r.owner = False
+            futures.extend(r._children)
 
-        if track:
-            tracker = zmq.MessageTracker(*trackers)
-        else:
-            tracker = None
-
-        r = AsyncResult(self.client, msg_ids, fname='scatter', targets=targets,
-            tracker=tracker, owner=True,
-        )
+        r = AsyncResult(self.client, futures, fname='scatter', targets=targets, owner=True)
         if block:
             r.wait()
         else:
@@ -784,10 +767,13 @@ class DirectView(View):
         # construct integer ID list:
         targets = self.client._build_targets(targets)[1]
         
+        futures = []
         for index, engineid in enumerate(targets):
-            msg_ids.extend(self.pull(key, block=False, targets=engineid).msg_ids)
+            ar = self.pull(key, block=False, targets=engineid)
+            ar.owner = False
+            futures.extend(ar._children)
 
-        r = AsyncMapResult(self.client, msg_ids, mapObject, fname='gather')
+        r = AsyncMapResult(self.client, futures, mapObject, fname='gather')
 
         if block:
             try:
@@ -1042,12 +1028,11 @@ class LoadBalancedView(View):
         follow = self._render_dependency(follow)
         metadata = dict(after=after, follow=follow, timeout=timeout, targets=idents, retries=retries)
 
-        msg = self.client.send_apply_request(self._socket, f, args, kwargs, track=track,
+        future = self.client.send_apply_request(self._socket, f, args, kwargs, track=track,
                                 metadata=metadata)
-        tracker = None if track is False else msg['tracker']
 
-        ar = AsyncResult(self.client, msg['header']['msg_id'], fname=getname(f),
-            targets=None, tracker=tracker, owner=True,
+        ar = AsyncResult(self.client, future, fname=getname(f),
+            targets=None, owner=True,
         )
         if block:
             try:
@@ -1118,4 +1103,35 @@ class LoadBalancedView(View):
         pf = ParallelFunction(self, f, block=block, chunksize=chunksize, ordered=ordered)
         return pf.map(*sequences)
 
-__all__ = ['LoadBalancedView', 'DirectView']
+from concurrent.futures import Executor
+
+class ViewExecutor(Executor):
+    """A PEP-3148 Executor API for Views
+    
+    Access as view.executor
+    """
+    def __init__(self, view):
+        self.view = view
+    
+    def submit(self, fn, *args, **kwargs):
+        """Same as View.apply_async"""
+        return self.view.apply_async(fn, *args, **kwargs)
+    
+    def map(self, func, *iterables, **kwargs):
+        """Return generator for View.map_async"""
+        if 'timeout' in kwargs:
+            warnings.warn("timeout unsupported in ViewExecutor.map")
+            kwargs.pop('timeout')
+        for r in self.view.map_async(func, *iterables, **kwargs):
+            yield r
+    
+    def shutdown(self, wait=True):
+        """ViewExecutor does *not* shutdown engines
+        
+        results are awaited if wait=True, but engines are *not* shutdown.
+        """
+        if wait:
+            self.view.wait()
+
+__all__ = ['LoadBalancedView', 'DirectView', 'ViewExecutor']
+
