@@ -5,18 +5,17 @@
 
 from __future__ import division
 
-import time
 from datetime import datetime
+import os
+import time
 
-import zmq
-
-import ipyparallel as parallel
+from IPython import get_ipython
 from ipyparallel.client import client as clientmod
 from ipyparallel import error
-from ipyparallel import AsyncResult, AsyncHubResult
-from ipyparallel import LoadBalancedView, DirectView
+from ipyparallel import AsyncHubResult
+from ipyparallel import DirectView
 
-from .clienttest import ClusterTestCase, segfault, wait, add_engines
+from .clienttest import ClusterTestCase, wait, add_engines
 
 def setup():
     add_engines(4, total=True)
@@ -206,14 +205,19 @@ class TestClient(ClusterTestCase):
 
     def test_shutdown(self):
         ids = self.client.ids
-        id0 = ids[0]
+        id0 = ids[-1]
+        pid = self.client[id0].apply_sync(os.getpid)
+        
         self.client.shutdown(id0, block=True)
-        while id0 in self.client.ids:
+        
+        for i in range(150):
+            # give the engine 15 seconds to die
+            if id0 not in self.client.ids:
+                break
             time.sleep(0.1)
-            self.client.spin()
-        
+        self.assertNotIn(id0, self.client.ids)
         self.assertRaises(IndexError, lambda : self.client[id0])
-        
+    
     def test_result_status(self):
         pass
         # to be written
@@ -436,39 +440,27 @@ class TestClient(ClusterTestCase):
         self.client.purge_hub_results(hist[-1])
         newhist = self.client.hub_history()
         self.assertEqual(len(newhist)+1,len(hist))
-        rc2.spin()
         rc2.close()
 
     def test_purge_local_results(self):
         # ensure there are some tasks
-        res = []
-        for i in range(5):
-            res.append(self.client[:].apply_async(lambda : 1))
-        self._wait_for_idle()
-        self.client.wait(10) # wait for the results to come back
+        # purge local results is mostly unnecessary now that we have Futures
+        msg_id = 'asdf'
+        self.client.results[msg_id] = 5
+        md = self.client.metadata[msg_id]
         before = len(self.client.results)
-        self.assertEqual(len(self.client.metadata),before)
-        self.client.purge_local_results(res[-1])
-        self.assertEqual(len(self.client.results),before-len(res[-1]), msg="Not removed from results")
-        self.assertEqual(len(self.client.metadata),before-len(res[-1]), msg="Not removed from metadata")
+        self.assertEqual(len(self.client.metadata), before)
+        self.client.purge_local_results(msg_id)
+        self.assertLessEqual(len(self.client.results), before-1, msg="Not removed from results")
+        self.assertLessEqual(len(self.client.metadata), before-1, msg="Not removed from metadata")
     
     def test_purge_local_results_outstanding(self):
         v = self.client[-1]
-        ar = v.apply_async(lambda : 1)
-        msg_id = ar.msg_ids[0]
-        ar.owner = False
-        ar.get()
-        self._wait_for_idle()
-        ar2 = v.apply_async(time.sleep, 1)
-        self.assertIn(msg_id, self.client.results)
-        self.assertIn(msg_id, self.client.metadata)
-        self.client.purge_local_results(ar)
-        self.assertNotIn(msg_id, self.client.results)
-        self.assertNotIn(msg_id, self.client.metadata)
+        ar = v.apply_async(time.sleep, 1)
         with self.assertRaises(RuntimeError):
-            self.client.purge_local_results(ar2)
-        ar2.get()
-        self.client.purge_local_results(ar2)
+            self.client.purge_local_results(ar)
+        ar.get()
+        self.client.purge_local_results(ar)
     
     def test_purge_all_local_results_outstanding(self):
         v = self.client[-1]
@@ -516,28 +508,6 @@ class TestClient(ClusterTestCase):
         # the hub results
         hist = self.client.hub_history()
         self.assertEqual(len(hist), 0, msg="hub history not empty")
-        
-    
-    def test_spin_thread(self):
-        self.client.spin_thread(0.01)
-        ar = self.client[-1].apply_async(lambda : 1)
-        md = self.client.metadata[ar.msg_ids[0]]
-        # 3s timeout, 100ms poll
-        for i in range(30):
-            time.sleep(0.1)
-            if md['received'] is not None:
-                break
-        self.assertIsInstance(md['received'], datetime)
-    
-    def test_stop_spin_thread(self):
-        self.client.spin_thread(0.01)
-        self.client.stop_spin_thread()
-        ar = self.client[-1].apply_async(lambda : 1)
-        md = self.client.metadata[ar.msg_ids[0]]
-        # 500ms timeout, 100ms poll
-        for i in range(5):
-            time.sleep(0.1)
-            self.assertIsNone(md['received'], None)
     
     def test_activate(self):
         ip = get_ipython()
